"""The test cases in this file comes from https://github.com/google/pyink,
which inherits Black's license (MIT license).
"""

# Literal list/tuple/set/dict will always be exploded on their own, immediately
# nested brackets should match this behavior.
literal_inner_set = [{
    1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012
}]

literal_inner_list = [[
    1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012
]]

literal_inner_tuple = [(
    1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012
)]

literal_inner_dict = [{
    1001: 1002, 1003: 1004, 1005: 1006, 1007: 1008, 1009: 1010, 1011: 1012
}]

literal_inner_multiple = [[[
    1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012
]]]


# Make sure trailing comma is still handled correctly.
some_var = [
    some_module_name.SomeClassName(arg_one="value_one", arg_two="value_two"),
    some_module_name.SomeClassName(arg_one="value_one", arg_two="value_two")
]


# output

"""The test cases in this file comes from https://github.com/google/pyink,
which inherits Black's license (MIT license).
"""

# Literal list/tuple/set/dict will always be exploded on their own, immediately
# nested brackets should match this behavior.
literal_inner_set = [{
    1001,
    1002,
    1003,
    1004,
    1005,
    1006,
    1007,
    1008,
    1009,
    1010,
    1011,
    1012,
}]

literal_inner_list = [[
    1001,
    1002,
    1003,
    1004,
    1005,
    1006,
    1007,
    1008,
    1009,
    1010,
    1011,
    1012,
]]

literal_inner_tuple = [(
    1001,
    1002,
    1003,
    1004,
    1005,
    1006,
    1007,
    1008,
    1009,
    1010,
    1011,
    1012,
)]

literal_inner_dict = [{
    1001: 1002,
    1003: 1004,
    1005: 1006,
    1007: 1008,
    1009: 1010,
    1011: 1012,
}]

literal_inner_multiple = [[[
    1001,
    1002,
    1003,
    1004,
    1005,
    1006,
    1007,
    1008,
    1009,
    1010,
    1011,
    1012,
]]]


# Make sure trailing comma is still handled correctly.
some_var = [
    some_module_name.SomeClassName(arg_one="value_one", arg_two="value_two"),
    some_module_name.SomeClassName(arg_one="value_one", arg_two="value_two"),
]



# These test cases demonstrate how line length calculation changes when
# we assign different widths to one tab. When tab width is small, some lines
# do not exceed the length limit. And when we increase tab width, the same
# lines start to exceed the limit and get wrapped.
if True:
	results = 1 + 2 + 3 + 4 + 5 + 6 + 7 + 8 + 9 + 10 + 11 + 12 + 13 + 14 + 15

	result = some_func(1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 90)

	class ClassWithLongName(ParentClassWithLongNameA, ParentClassWithLongNameB_):
		def __init__(self):
			pass

	def inner_func_with_very_long_name(arg1_with_longname, arg_2_with_long_name_):
		pass

	result = inner_func_with_very_long_name(123456789123456789, 'abcdefghijklmnop')


# output

# These test cases demonstrate how line length calculation changes when
# we assign different widths to one tab. When tab width is small, some lines
# do not exceed the length limit. And when we increase tab width, the same
# lines start to exceed the limit and get wrapped.
if True:
	results = 1 + 2 + 3 + 4 + 5 + 6 + 7 + 8 + 9 + 10 + 11 + 12 + 13 + 14 + 15

	result = some_func(1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 90)

	class ClassWithLongName(ParentClassWithLongNameA, ParentClassWithLongNameB_):
		def __init__(self):
			pass

	def inner_func_with_very_long_name(
		arg1_with_longname, arg_2_with_long_name_
	):
		pass

	result = inner_func_with_very_long_name(
		123456789123456789, 'abcdefghijklmnop'
	)



# --function-definition-extra-indent = True
# --other-line-continuation-extra-indent = False
def function(
      arg1_with_very_long_name: int,
      arg2_with_very_long_name: float,
      arg3_with_very_long_name: list,
):
   print('Hello world')


def func2(
      arg1_with_very_long_name: int,
      arg2_with_very_long_name: float,
      arg3_with_very_long_name: list,
) -> None:
   print('Hello world')


def func3(a, b, c, d, e) -> int:
   return 2


def outer(
      arg1_with_very_long_name: int,
      *,
      arg2_with_very_long_name: float,
      arg3_with_very_long_name: list,
) -> int:
   def middle(
         arg1_with_very_long_name: int,
         arg2_with_very_long_name: float,
         arg3_with_very_long_name: list,
         *args,
   ):
      def inner(
            arg1_with_very_long_name: int,
            arg2_with_very_long_name: float,
            arg3_with_very_long_name: list,
            *args,
            **kwargs,
      ):
         return 1

      print('middle layer')
      return 2

   return 3


class A:
   def __init__(
         self,
         arg1_with_very_long_name: int,
         arg2_with_very_long_name: float,
         *,
         arg3_with_very_long_name: list,
   ) -> None:
      print('hello world')

   def outer(
         self,
         arg1_with_very_long_name: int,
         *,
         arg2_with_very_long_name: float,
         arg3_with_very_long_name: list,
   ) -> int:
      def middle(
            arg1_with_very_long_name: int,
            arg2_with_very_long_name: float,
            arg3_with_very_long_name: list,
            *args,
      ):
         def inner(
               arg1_with_very_long_name: int,
               arg2_with_very_long_name: float,
               arg3_with_very_long_name: list,
               *args,
               **kwargs,
         ):
            class B:
               def __init__(
                     self,
                     arg1_with_very_long_name: int,
                     arg2_with_very_long_name: float,
                     arg3_with_very_long_name: list,
                     *args,
                     **kwargs,
               ):
                  print('initialized')

            return 1

         print('middle layer')
         return 2

      return 3


aa = [
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
]


bb = func1(
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
)


cc = {
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   123123123123123123123123123123123123123123123123123,
   'value_with_very_very_very_very_very_very_long_name',
}


if (
   value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
):
   print(123)


some_val_with_very_long_name = (
   'very very very long string' + 'this is an even longer string'
)



from dataclasses import dataclass
from typing import TYPE_CHECKING, Callable, List, Optional, Type

from blib2to3.pgen2 import token
from blib2to3.pytree import Leaf
from cercis.lines import Line
from cercis.nodes import CLOSING_BRACKETS, OPENING_BRACKETS

if TYPE_CHECKING:
    from cercis.linegen import _BracketSplitComponent
    from cercis.mode import Mode


@dataclass
class CollapseNestedBracketsReturnValues:
    head_leaves: List[Leaf]
    body_leaves: List[Leaf]
    tail_leaves: List[Leaf]
    body: Optional[Line]


def perform_collapse_nested_brackets(
        *,
        line: Line,
        opening_bracket: Leaf,
        closing_bracket: Leaf,
        head_leaves: List[Leaf],
        body_leaves: List[Leaf],
        tail_leaves: List[Leaf],
        mode: "Mode",
        bracket_split_build_line_func: Callable[..., Line],
        bracket_split_component: "Type[_BracketSplitComponent]",
) -> CollapseNestedBracketsReturnValues:
    """
    The code of this function comes from a wonderful implementation
    from another Black fork: https://github.com/google/pyink/blob/f93771c02e9a26ce9508c59d69c9337c95797eac/src/pyink/linegen.py#L778-L815  # noqa: B950

    Pyink inherits the license of Black, which is already included in this
    repo (the file OLD_LICENSE).
    """
    opening_brackets: List[Leaf] = [opening_bracket]
    body: Optional[Line] = None
    if mode.collapse_nested_brackets and not (
        # Only look inside when it doesn't start with invisible parens.
        opening_bracket.type == token.LPAR
        and not opening_bracket.value
        and closing_bracket.type == token.RPAR
        and not closing_bracket.value
    ):
        # Find an inner body...
        inner_body_leaves = list(body_leaves)
        inner_opening_brackets: List[Leaf] = []
        inner_closing_brackets: List[Leaf] = []
        while (
            len(inner_body_leaves) >= 2
            and inner_body_leaves[0].type in OPENING_BRACKETS
            and inner_body_leaves[-1].type in CLOSING_BRACKETS
            and inner_body_leaves[-1].opening_bracket is inner_body_leaves[0]
        ):
            inner_opening_brackets.append(inner_body_leaves.pop(0))
            inner_closing_brackets.insert(0, inner_body_leaves.pop())
        if len(inner_body_leaves) < len(body_leaves):
            inner_body = bracket_split_build_line_func(
                inner_body_leaves,
                line,
                opening_brackets[0],
                component=bracket_split_component.body,
                mode=mode,
            )
            if inner_body.should_split_rhs or (
                inner_body_leaves and inner_body_leaves[-1].type == token.COMMA
            ):
                # Only when the inner body itself will be split or ends with a comma,
                # should we prefer not break immediately nested brackets.
                body_leaves = inner_body_leaves
                head_leaves.extend(inner_opening_brackets)
                tail_leaves = inner_closing_brackets + tail_leaves
                body = inner_body  # No need to re-calculate body.

    return CollapseNestedBracketsReturnValues(
        head_leaves=head_leaves,
        body_leaves=body_leaves,
        tail_leaves=tail_leaves,
        body=body,
    )



# --function-definition-extra-indent = False
# --other-line-continuation-extra-indent = True
def function(
        arg1_with_very_long_name: int,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
        ):
        print('Hello world')


def func2(
        arg1_with_very_long_name: int,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
        ) -> None:
        print('Hello world')


def func3(a, b, c, d, e) -> int:
        return 2


def outer(
        arg1_with_very_long_name: int,
        *,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
        ) -> int:
        def middle(
                arg1_with_very_long_name: int,
                arg2_with_very_long_name: float,
                arg3_with_very_long_name: list,
                *args,
                ):
                def inner(
                        arg1_with_very_long_name: int,
                        arg2_with_very_long_name: float,
                        arg3_with_very_long_name: list,
                        *args,
                        **kwargs,
                        ):
                        return 1

                print('middle layer')
                return 2

        return 3


class A:
        def __init__(
                self,
                arg1_with_very_long_name: int,
                arg2_with_very_long_name: float,
                *,
                arg3_with_very_long_name: list,
                ) -> None:
                print('hello world')

        def outer(
                self,
                arg1_with_very_long_name: int,
                *,
                arg2_with_very_long_name: float,
                arg3_with_very_long_name: list,
                ) -> int:
                def middle(
                        arg1_with_very_long_name: int,
                        arg2_with_very_long_name: float,
                        arg3_with_very_long_name: list,
                        *args,
                        ):
                        def inner(
                                arg1_with_very_long_name: int,
                                arg2_with_very_long_name: float,
                                arg3_with_very_long_name: list,
                                *args,
                                **kwargs,
                                ):
                                class B:
                                        def __init__(
                                                self,
                                                arg1_with_very_long_name: int,
                                                arg2_with_very_long_name: float,
                                                arg3_with_very_long_name: list,
                                                *args,
                                                **kwargs,
                                                ):
                                                print('initialized')

                                return 1

                        print('middle layer')
                        return 2

                return 3


aa = [
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                ]


bb = func1(
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                )


cc = {
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                value_with_very_very_very_very_very_very_long_name,
                123123123123123123123123123123123123123123123123123,
                'value_with_very_very_very_very_very_very_long_name',
                }


if (
                value_with_very_very_very_very_very_very_long_name
                and value_with_very_very_very_very_very_very_long_name
                and value_with_very_very_very_very_very_very_long_name
                and value_with_very_very_very_very_very_very_long_name
                and value_with_very_very_very_very_very_very_long_name
                and value_with_very_very_very_very_very_very_long_name
                and value_with_very_very_very_very_very_very_long_name
                ):
        print(123)


some_val_with_very_long_name = (
                'very very very long string' + 'this is an even longer string'
                )



# --function-definition-extra-indent = False
# --other-line-continuation-extra-indent = False
def function(
 arg1_with_very_long_name: int,
 arg2_with_very_long_name: float,
 arg3_with_very_long_name: list,
):
 print('Hello world')


def func2(
 arg1_with_very_long_name: int,
 arg2_with_very_long_name: float,
 arg3_with_very_long_name: list,
) -> None:
 print('Hello world')


def func3(a, b, c, d, e) -> int:
 return 2


def outer(
 arg1_with_very_long_name: int,
 *,
 arg2_with_very_long_name: float,
 arg3_with_very_long_name: list,
) -> int:
 def middle(
  arg1_with_very_long_name: int,
  arg2_with_very_long_name: float,
  arg3_with_very_long_name: list,
  *args,
 ):
  def inner(
   arg1_with_very_long_name: int,
   arg2_with_very_long_name: float,
   arg3_with_very_long_name: list,
   *args,
   **kwargs,
  ):
   return 1

  print('middle layer')
  return 2

 return 3


class A:
 def __init__(
  self,
  arg1_with_very_long_name: int,
  arg2_with_very_long_name: float,
  *,
  arg3_with_very_long_name: list,
 ) -> None:
  print('hello world')

 def outer(
  self,
  arg1_with_very_long_name: int,
  *,
  arg2_with_very_long_name: float,
  arg3_with_very_long_name: list,
 ) -> int:
  def middle(
   arg1_with_very_long_name: int,
   arg2_with_very_long_name: float,
   arg3_with_very_long_name: list,
   *args,
  ):
   def inner(
    arg1_with_very_long_name: int,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
    *args,
    **kwargs,
   ):
    class B:
     def __init__(
      self,
      arg1_with_very_long_name: int,
      arg2_with_very_long_name: float,
      arg3_with_very_long_name: list,
      *args,
      **kwargs,
     ):
      print('initialized')

    return 1

   print('middle layer')
   return 2

  return 3


aa = [
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
]


bb = func1(
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
)


cc = {
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 value_with_very_very_very_very_very_very_long_name,
 123123123123123123123123123123123123123123123123123,
 'value_with_very_very_very_very_very_very_long_name',
}


if (
 value_with_very_very_very_very_very_very_long_name
 and value_with_very_very_very_very_very_very_long_name
 and value_with_very_very_very_very_very_very_long_name
 and value_with_very_very_very_very_very_very_long_name
 and value_with_very_very_very_very_very_very_long_name
 and value_with_very_very_very_very_very_very_long_name
 and value_with_very_very_very_very_very_very_long_name
):
 print(123)


some_val_with_very_long_name = (
 'very very very long string' + 'this is an even longer string'
)



# The input source must not contain any Py36-specific syntax (e.g. argument type
# annotations, trailing comma after *rest) or this test becomes invalid.
def long_function_name(argument_one, argument_two, argument_three, argument_four, argument_five, argument_six, *rest): ...
# output
# The input source must not contain any Py36-specific syntax (e.g. argument type
# annotations, trailing comma after *rest) or this test becomes invalid.
def long_function_name(
        argument_one,
        argument_two,
        argument_three,
        argument_four,
        argument_five,
        argument_six,
        *rest,
):
    ...



# --function-definition-extra-indent = False
# --other-line-continuation-extra-indent = False
def function(
	arg1_with_very_long_name: int,
	arg2_with_very_long_name: float,
	arg3_with_very_long_name: list,
	):
	print('Hello world')


def func2(
	arg1_with_very_long_name: int,
	arg2_with_very_long_name: float,
	arg3_with_very_long_name: list,
	) -> None:
	print('Hello world')


def func3(a, b, c, d, e) -> int:
	return 2


def outer(
	arg1_with_very_long_name: int,
	*,
	arg2_with_very_long_name: float,
	arg3_with_very_long_name: list,
	) -> int:
	def middle(
		arg1_with_very_long_name: int,
		arg2_with_very_long_name: float,
		arg3_with_very_long_name: list,
		*args,
		):
		def inner(
			arg1_with_very_long_name: int,
			arg2_with_very_long_name: float,
			arg3_with_very_long_name: list,
			*args,
			**kwargs,
			):
			return 1

		print('middle layer')
		return 2

	return 3


class A:
	def __init__(
		self,
		arg1_with_very_long_name: int,
		arg2_with_very_long_name: float,
		*,
		arg3_with_very_long_name: list,
		) -> None:
		print('hello world')

	def outer(
		self,
		arg1_with_very_long_name: int,
		*,
		arg2_with_very_long_name: float,
		arg3_with_very_long_name: list,
		) -> int:
		def middle(
			arg1_with_very_long_name: int,
			arg2_with_very_long_name: float,
			arg3_with_very_long_name: list,
			*args,
			):
			def inner(
				arg1_with_very_long_name: int,
				arg2_with_very_long_name: float,
				arg3_with_very_long_name: list,
				*args,
				**kwargs,
				):
				class B:
					def __init__(
						self,
						arg1_with_very_long_name: int,
						arg2_with_very_long_name: float,
						arg3_with_very_long_name: list,
						*args,
						**kwargs,
						):
						print('initialized')

				return 1

			print('middle layer')
			return 2

		return 3


aa = [
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	]


bb = func1(
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	)


cc = {
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	value_with_very_very_very_very_very_very_long_name,
	123123123123123123123123123123123123123123123123123,
	'value_with_very_very_very_very_very_very_long_name',
	}


if (
	value_with_very_very_very_very_very_very_long_name
	and value_with_very_very_very_very_very_very_long_name
	and value_with_very_very_very_very_very_very_long_name
	and value_with_very_very_very_very_very_very_long_name
	and value_with_very_very_very_very_very_very_long_name
	and value_with_very_very_very_very_very_very_long_name
	and value_with_very_very_very_very_very_very_long_name
	):
	print(123)


some_val_with_very_long_name = (
	'very very very long string' + 'this is an even longer string'
	)



DEFAULT_LINE_LENGTH = 79
DEFAULT_EXCLUDES = r"/(\.direnv|\.eggs|\.git|\.hg|\.mypy_cache|\.nox|\.tox|\.venv|venv|\.svn|\.ipynb_checkpoints|_build|buck-out|build|dist|__pypackages__)/"  # noqa: B950
DEFAULT_INCLUDES = r"(\.pyi?|\.ipynb)$"
STDIN_PLACEHOLDER = "__BLACK_STDIN_FILENAME__"
DEFAULT_FUNCTION_DEFINITION_EXTRA_INDENT = True
DEFAULT_CLOSING_BRACKET_EXTRA_INDENT = False
DEFAULT_SINGLE_QUOTE = True
DEFAULT_WRAP_LINE_WITH_LONG_STRING = False
DEFAULT_COLLAPSE_NESTED_BRACKETS = True
DEFAULT_WRAP_PRAGMA_COMMENTS = False
DEFAULT_BASE_INDENTATION_SPACES = 4
DEFAULT_OTHER_LINE_CONTINUATION_EXTRA_INDENT = False
DEFAULT_USE_TABS = False
DEFAULT_TAB_WIDTH = 4



# Some test cases in this file comes from: https://github.com/google/pyink/blob/f93771c02e9a26ce9508c59d69c9337c95797eac/tests/data/pyink/pragma_comments.py
a_very_long_library_name._private_method(and_a_long_arg)  # pylint: disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pylint: disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pylint: disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # pylint: disable=protected-access

a_very_long_library_name._private_method(and_a_long_arg)  # pylint:disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pylint:disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pylint:disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # pylint:disable=protected-access

a_very_long_library_name._private_method(and_a_long_arg)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # noqa: E123, W234, ABC456, XYZ2

a_very_long_library_name._private_method(and_a_long_arg)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # noqa:E123, W234, ABC456, XYZ2

# Black does not wrap a line, no matter how long,
# if it has "# type: ignore" at the end.
a_very_long_library_name._private_method(and_a_long_arg)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # type: ignore[something]
a_very_long_library_name._private_method(Though_hundreds_of_thousands_had_done_their_very_best_to_disfigure_the_small_piece_of_land_on_which_they_were_crowded_together_by_paving_the_ground_with_stones_scraping_away_every_vestige_of_vegetation_cutting_down_the_trees_turning_away_birds_and_beasts_and_filling_the_air_with_the_smoke_of_naphtha_and_coal_still_spring_was_spring_even_in_the_town)  # type: ignore[something]

# Black only exempts "# type: ignore" but not "# type:ignore".
# The latter is also valid for mypy and many people use the latter.
a_very_long_library_name._private_method(and_a_long_arg)  # type:ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type:ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type:ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # type:ignore[something]
a_very_long_library_name._private_method(Though_hundreds_of_thousands_had_done_their_very_best_to_disfigure_the_small_piece_of_land_on_which_they_were_crowded_together_by_paving_the_ground_with_stones_scraping_away_every_vestige_of_vegetation_cutting_down_the_trees_turning_away_birds_and_beasts_and_filling_the_air_with_the_smoke_of_naphtha_and_coal_still_spring_was_spring_even_in_the_town)  # type:ignore[something]

# output

# Some test cases in this file comes from: https://github.com/google/pyink/blob/f93771c02e9a26ce9508c59d69c9337c95797eac/tests/data/pyink/pragma_comments.py
a_very_long_library_name._private_method(
    and_a_long_arg
)  # pylint: disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit_
)  # pylint: disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit___
)  # pylint: disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # pylint: disable=protected-access

a_very_long_library_name._private_method(
    and_a_long_arg
)  # pylint:disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit_
)  # pylint:disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit___
)  # pylint:disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # pylint:disable=protected-access

a_very_long_library_name._private_method(
    and_a_long_arg
)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit_
)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit___
)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # noqa: E123, W234, ABC456, XYZ2

a_very_long_library_name._private_method(
    and_a_long_arg
)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit_
)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit___
)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # noqa:E123, W234, ABC456, XYZ2

# Black does not wrap a line, no matter how long,
# if it has "# type: ignore" at the end.
a_very_long_library_name._private_method(and_a_long_arg)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # type: ignore[something]
a_very_long_library_name._private_method(Though_hundreds_of_thousands_had_done_their_very_best_to_disfigure_the_small_piece_of_land_on_which_they_were_crowded_together_by_paving_the_ground_with_stones_scraping_away_every_vestige_of_vegetation_cutting_down_the_trees_turning_away_birds_and_beasts_and_filling_the_air_with_the_smoke_of_naphtha_and_coal_still_spring_was_spring_even_in_the_town)  # type: ignore[something]

# Black only exempts "# type: ignore" but not "# type:ignore".
# The latter is also valid for mypy and many people use the latter.
a_very_long_library_name._private_method(
    and_a_long_arg
)  # type:ignore[something]
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit_
)  # type:ignore[something]
a_very_long_library_name._private_method(
    and_a_long_arg_that_just_fits_limit___
)  # type:ignore[something]
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # type:ignore[something]
a_very_long_library_name._private_method(
    Though_hundreds_of_thousands_had_done_their_very_best_to_disfigure_the_small_piece_of_land_on_which_they_were_crowded_together_by_paving_the_ground_with_stones_scraping_away_every_vestige_of_vegetation_cutting_down_the_trees_turning_away_birds_and_beasts_and_filling_the_air_with_the_smoke_of_naphtha_and_coal_still_spring_was_spring_even_in_the_town
)  # type:ignore[something]



from typing import List

from blib2to3.pgen2 import token
from blib2to3.pytree import Leaf
from cercis.lines import Line


def check_eligibility_to_opt_out_of_line_wrapping(
        line: Line,
        wrap_line_with_long_string: bool,
) -> bool:
    """
    Only "simple" lines are eligible for opting out of line wrapping.

    Some examples of "simple" lines:
        - var = "yes"
        - var = "hello"     " world"
        - var = "very very very very very very very very very long string"
        - var.attr.c.d = "very very very very very very very very long string"

    The last element of the line, except for the comment, must be a string.

    Some examples of "non-simple" lines:
        - var = "hello" + "world"
        - lookUp["key"] = "hello world"

    For a more comprehensive list of examples, go to `test_lingen.py` and
    check out the test cases.
    """
    if wrap_line_with_long_string:  # comes from top-level config
        return False  # not eligible (always use Black's default behavior)

    if len(line.comments) > 0:
        # As long as there are any comments, we don't consider this `line`
        # eligible for opting out, i.e., we fall back to Black's default
        # behaviors.
        #
        # If we don't fall back, there will be conflicts with Black's
        # experimental features ("--preview").
        return False

    if len(line.leaves) == 0:
        return False

    # LPAR and RPAR are added during node visiting (in `lines.visit(src_node)`)
    if line.leaves[-1].type == token.RPAR:
        leaves = _clone_leaves(line.leaves[:-1])
        has_rpar = True
    else:
        leaves = _clone_leaves(line.leaves)
        has_rpar = False

    if len(leaves) == 0 or not _is_string(leaves[-1]):
        return False

    while len(leaves) > 0:  # check leaf by leaf from right hand side
        leaves = leaves[:-1]
        if len(leaves) == 0:
            break  # note: all([]) is True

        if _is_string(leaves[-1]):
            continue

        if has_rpar and leaves[-1].type == token.LPAR:
            has_rpar = False
            continue

        if not _is_equal_operators(leaves[-1]):
            return False

        leaves = leaves[:-1]
        break

    return all(_is_name_or_dot(_) for _ in leaves)


def _clone_leaves(leaves: List[Leaf]) -> List[Leaf]:
    return [_.clone() for _ in leaves]


def _is_string(leaf: Leaf) -> bool:
    return leaf.type == token.STRING


def _is_equal_operators(leaf: Leaf) -> bool:
    if leaf.type == token.EQUAL:
        return True

    return 37 <= leaf.type <= 47


def _is_name_or_dot(leaf: Leaf) -> bool:
    return leaf.type in {token.NAME, token.DOT}



# --function-definition-extra-indent = True
# --other-line-continuation-extra-indent = True
def function(
    arg1_with_very_long_name: int,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
    ):
  print('Hello world')


def func2(
    arg1_with_very_long_name: int,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
    ) -> None:
  print('Hello world')


def func3(a, b, c, d, e) -> int:
  return 2


def outer(
    arg1_with_very_long_name: int,
    *,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
    ) -> int:
  def middle(
      arg1_with_very_long_name: int,
      arg2_with_very_long_name: float,
      arg3_with_very_long_name: list,
      *args,
      ):
    def inner(
        arg1_with_very_long_name: int,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
        *args,
        **kwargs,
        ):
      return 1

    print('middle layer')
    return 2

  return 3


class A:
  def __init__(
      self,
      arg1_with_very_long_name: int,
      arg2_with_very_long_name: float,
      *,
      arg3_with_very_long_name: list,
      ) -> None:
    print('hello world')

  def outer(
      self,
      arg1_with_very_long_name: int,
      *,
      arg2_with_very_long_name: float,
      arg3_with_very_long_name: list,
      ) -> int:
    def middle(
        arg1_with_very_long_name: int,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
        *args,
        ):
      def inner(
          arg1_with_very_long_name: int,
          arg2_with_very_long_name: float,
          arg3_with_very_long_name: list,
          *args,
          **kwargs,
          ):
        class B:
          def __init__(
              self,
              arg1_with_very_long_name: int,
              arg2_with_very_long_name: float,
              arg3_with_very_long_name: list,
              *args,
              **kwargs,
              ):
            print('initialized')

        return 1

      print('middle layer')
      return 2

    return 3


aa = [
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    ]


bb = func1(
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    )


cc = {
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    123123123123123123123123123123123123123123123123123,
    'value_with_very_very_very_very_very_very_long_name',
    }


if (
    value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    ):
  print(123)


some_val_with_very_long_name = (
    'very very very long string' + 'this is an even longer string'
    )



# --function-definition-extra-indent = True
# --other-line-continuation-extra-indent = True
def function(
  arg1_with_very_long_name: int,
  arg2_with_very_long_name: float,
  arg3_with_very_long_name: list,
  ):
 print('Hello world')


def func2(
  arg1_with_very_long_name: int,
  arg2_with_very_long_name: float,
  arg3_with_very_long_name: list,
  ) -> None:
 print('Hello world')


def func3(a, b, c, d, e) -> int:
 return 2


def outer(
  arg1_with_very_long_name: int,
  *,
  arg2_with_very_long_name: float,
  arg3_with_very_long_name: list,
  ) -> int:
 def middle(
   arg1_with_very_long_name: int,
   arg2_with_very_long_name: float,
   arg3_with_very_long_name: list,
   *args,
   ):
  def inner(
    arg1_with_very_long_name: int,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
    *args,
    **kwargs,
    ):
   return 1

  print('middle layer')
  return 2

 return 3


class A:
 def __init__(
   self,
   arg1_with_very_long_name: int,
   arg2_with_very_long_name: float,
   *,
   arg3_with_very_long_name: list,
   ) -> None:
  print('hello world')

 def outer(
   self,
   arg1_with_very_long_name: int,
   *,
   arg2_with_very_long_name: float,
   arg3_with_very_long_name: list,
   ) -> int:
  def middle(
    arg1_with_very_long_name: int,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
    *args,
    ):
   def inner(
     arg1_with_very_long_name: int,
     arg2_with_very_long_name: float,
     arg3_with_very_long_name: list,
     *args,
     **kwargs,
     ):
    class B:
     def __init__(
       self,
       arg1_with_very_long_name: int,
       arg2_with_very_long_name: float,
       arg3_with_very_long_name: list,
       *args,
       **kwargs,
       ):
      print('initialized')

    return 1

   print('middle layer')
   return 2

  return 3


aa = [
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  ]


bb = func1(
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  )


cc = {
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  value_with_very_very_very_very_very_very_long_name,
  123123123123123123123123123123123123123123123123123,
  'value_with_very_very_very_very_very_very_long_name',
  }


if (
  value_with_very_very_very_very_very_very_long_name
  and value_with_very_very_very_very_very_very_long_name
  and value_with_very_very_very_very_very_very_long_name
  and value_with_very_very_very_very_very_very_long_name
  and value_with_very_very_very_very_very_very_long_name
  and value_with_very_very_very_very_very_very_long_name
  and value_with_very_very_very_very_very_very_long_name
  ):
 print(123)


some_val_with_very_long_name = (
  'very very very long string' + 'this is an even longer string'
  )



class Outer:
    class InnerStub: ...
    outer_attr_after_inner_stub: int
    class Inner:
        inner_attr: int
    outer_attr: int

# output
class Outer:
    class InnerStub: ...
    outer_attr_after_inner_stub: int

    class Inner:
        inner_attr: int

    outer_attr: int



import json
import re
import sys

import requests  # type:ignore

try:
    from packaging.version import Version, parse
except ImportError:
    from pip._vendor.packaging.version import Version, parse  # type:ignore


PATTERN = re.compile(r'\[(.*?)\]')
VER_PAT = re.compile(r'^\d\.\d\.\d$')

URL_PATTERN = 'https://pypi.python.org/pypi/{package}/json'


def get_latest_version_from_PyPI(
        package: str,
        url_pattern: str = URL_PATTERN,
) -> Version:
    """Return version of package on pypi.python.org using json."""
    req = requests.get(url_pattern.format(package=package))
    version = parse('0')
    if req.status_code == requests.codes.ok:
        j = json.loads(req.text.encode(req.encoding))
        releases = j.get('releases', [])
        for release in releases:
            ver = parse(release)
            if not ver.is_prerelease:
                version = max(version, ver)

    return version


def should_fail_version_check(
        mode: str,
        local_latest_ver: Version,
        pypi_latest_ver: Version,
) -> bool:
    if mode == 'branch':
        return local_latest_ver < pypi_latest_ver

    if mode == 'main':
        return local_latest_ver <= pypi_latest_ver

    raise RuntimeError('Internal error; please contact the authors')


def main() -> int:
    mode = sys.argv[1] if len(sys.argv) > 1 else 'branch'

    with open('CHANGELOG.md') as fp:
        lines = fp.readlines()

    if len(lines) == 0:
        print('Empty CHANGELOG.md file')
        return 1

    print('Contents of CHANGELOG.md (up to the latest local version):\n')
    if lines[0] != '# Change Log\n':
        print(f'Unexpected first line in CHANGELOG.md: {lines[0]}')
        return 1

    for line in lines[1:]:
        print(line.rstrip())
        if line.startswith('##'):
            if line == '## [Unreleased]\n':
                continue

            match = PATTERN.search(line)
            version = match.group(1)  # type:ignore
            if not VER_PAT.match(version):
                print(f'Invalid version: {version}')
                return 1

            break

    latest_version_on_PyPI = get_latest_version_from_PyPI('cercis')

    if should_fail_version_check(mode, parse(version), latest_version_on_PyPI):
        print(
            f'Newest local version ({version}) must be higher than the'
            f' latest version on PyPI ({latest_version_on_PyPI})'
        )
        return 1

    return 0


if __name__ == '__main__':
    raise SystemExit(main())



"""The test cases in this file comes from https://github.com/google/pyink,
which inherits Black's license (MIT license).
"""

# Literal list/tuple/set/dict will always be exploded on their own, immediately
# nested brackets should match this behavior.
literal_inner_set = [{
    1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012
}]

literal_inner_list = [[
    1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012
]]

literal_inner_tuple = [(
    1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012
)]

literal_inner_dict = [{
    1001: 1002, 1003: 1004, 1005: 1006, 1007: 1008, 1009: 1010, 1011: 1012
}]

literal_inner_multiple = [[[
    1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012
]]]


# Make sure trailing comma is still handled correctly.
some_var = [
    some_module_name.SomeClassName(arg_one="value_one", arg_two="value_two"),
    some_module_name.SomeClassName(arg_one="value_one", arg_two="value_two")
]


# output

"""The test cases in this file comes from https://github.com/google/pyink,
which inherits Black's license (MIT license).
"""

# Literal list/tuple/set/dict will always be exploded on their own, immediately
# nested brackets should match this behavior.
literal_inner_set = [
    {1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012}
]

literal_inner_list = [
    [1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012]
]

literal_inner_tuple = [
    (1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012)
]

literal_inner_dict = [
    {1001: 1002, 1003: 1004, 1005: 1006, 1007: 1008, 1009: 1010, 1011: 1012}
]

literal_inner_multiple = [
    [[1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012]]
]


# Make sure trailing comma is still handled correctly.
some_var = [
    some_module_name.SomeClassName(arg_one="value_one", arg_two="value_two"),
    some_module_name.SomeClassName(arg_one="value_one", arg_two="value_two"),
]



# Some test cases in this file comes from: https://github.com/google/pyink/blob/f93771c02e9a26ce9508c59d69c9337c95797eac/tests/data/pyink/pragma_comments.py
a_very_long_library_name._private_method(and_a_long_arg)  # pylint:disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pylint:disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pylint:disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # pylint:disable=protected-access

a_very_long_library_name._private_method(and_a_long_arg)  # pylint: disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pylint: disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pylint: disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # pylint: disable=protected-access

a_very_long_library_name._private_method(and_a_long_arg)  # pylint:  disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pylint:  disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pylint:  disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # pylint:  disable=protected-access

a_very_long_library_name._private_method(and_a_long_arg)  # pytype:disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pytype:disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pytype:disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # pytype:disable=attribute-error

a_very_long_library_name._private_method(and_a_long_arg)  # pytype: disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pytype: disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pytype: disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # pytype: disable=attribute-error

a_very_long_library_name._private_method(and_a_long_arg)  # pytype:  disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pytype:  disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pytype:  disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # pytype:  disable=attribute-error

a_very_long_library_name._private_method(and_a_long_arg)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # noqa:E123, W234, ABC456, XYZ2

a_very_long_library_name._private_method(and_a_long_arg)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # noqa: E123, W234, ABC456, XYZ2

a_very_long_library_name._private_method(and_a_long_arg)  # noqa:  E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # noqa:  E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # noqa:  E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # noqa:  E123, W234, ABC456, XYZ2

a_very_long_library_name._private_method(and_a_long_arg)  # type:ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type:ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type:ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # type:ignore[something]

a_very_long_library_name._private_method(and_a_long_arg)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # type: ignore[something]

a_very_long_library_name._private_method(and_a_long_arg)  # type:  ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type:  ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type:  ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_no_long_fits_______)  # type:  ignore[something]

# output

# Some test cases in this file comes from: https://github.com/google/pyink/blob/f93771c02e9a26ce9508c59d69c9337c95797eac/tests/data/pyink/pragma_comments.py
a_very_long_library_name._private_method(and_a_long_arg)  # pylint:disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pylint:disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pylint:disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # pylint:disable=protected-access

a_very_long_library_name._private_method(and_a_long_arg)  # pylint: disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pylint: disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pylint: disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # pylint: disable=protected-access

a_very_long_library_name._private_method(and_a_long_arg)  # pylint:  disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pylint:  disable=protected-access
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pylint:  disable=protected-access
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # pylint:  disable=protected-access

a_very_long_library_name._private_method(and_a_long_arg)  # pytype:disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pytype:disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pytype:disable=attribute-error
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # pytype:disable=attribute-error

a_very_long_library_name._private_method(and_a_long_arg)  # pytype: disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pytype: disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pytype: disable=attribute-error
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # pytype: disable=attribute-error

a_very_long_library_name._private_method(and_a_long_arg)  # pytype:  disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # pytype:  disable=attribute-error
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # pytype:  disable=attribute-error
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # pytype:  disable=attribute-error

a_very_long_library_name._private_method(and_a_long_arg)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # noqa:E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # noqa:E123, W234, ABC456, XYZ2

a_very_long_library_name._private_method(and_a_long_arg)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # noqa: E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # noqa: E123, W234, ABC456, XYZ2

a_very_long_library_name._private_method(and_a_long_arg)  # noqa:  E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # noqa:  E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # noqa:  E123, W234, ABC456, XYZ2
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # noqa:  E123, W234, ABC456, XYZ2

a_very_long_library_name._private_method(and_a_long_arg)  # type:ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type:ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type:ignore[something]
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # type:ignore[something]

a_very_long_library_name._private_method(and_a_long_arg)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type: ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type: ignore[something]
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # type: ignore[something]

a_very_long_library_name._private_method(and_a_long_arg)  # type:  ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit_)  # type:  ignore[something]
a_very_long_library_name._private_method(and_a_long_arg_that_just_fits_limit___)  # type:  ignore[something]
a_very_long_library_name._private_method(
    and_a_long_arg_that_no_long_fits_______
)  # type:  ignore[something]



long_unmergable_string_with_pragma = (
    "This is a really long string that can't be merged because it has a likely pragma at the end"  # type: ignore
    " of it."
)

long_unmergable_string_with_pragma = (
    "This is a really long string that can't be merged because it has a likely pragma at the end"  # noqa
    " of it."
)

long_unmergable_string_with_pragma = (
    "This is a really long string that can't be merged because it has a likely pragma at the end"  # pylint: disable=some-pylint-check
    " of it."
)

long_unmergable_string_with_comment = (
    "This is a really long string that can't be merged because it has a likely pragma at the end"  # hello world
    " of it."
)

long_unmergable_string_with_comment = (
    "This is a really long string that can't be merged because it has a likely pragma at the end"
    " of it."  # good morning
)



def positional_only_arg(a, /):
    pass


def all_markers(a, b, /, c, d, *, e, f):
    pass


def all_markers_with_args_and_kwargs(
        a_long_one,
        b_long_one,
        /,
        c_long_one,
        d_long_one,
        *args,
        e_long_one,
        f_long_one,
        **kwargs,
):
    pass


def all_markers_with_defaults(a, b=1, /, c=2, d=3, *, e=4, f=5):
    pass


def long_one_with_long_parameter_names(
        but_all_of_them,
        are_positional_only,
        arguments_mmmmkay,
        so_this_is_only_valid_after,
        three_point_eight,
        /,
):
    pass


lambda a, /: a

lambda a, b, /, c, d, *, e, f: a

lambda a, b, /, c, d, *args, e, f, **kwargs: args

lambda a, b=1, /, c=2, d=3, *, e=4, f=5: 1



from dataclasses import dataclass

variable_1 = "very very very very very very very very very very very very very very long string"

variable_2 = "very very very very very very very very very very very very very very very long string"

variable_3 = 'very very very very very very very very very very very very very very long string'


@dataclass
class E:
    f: str = ""


@dataclass
class D:
    e: E = None


@dataclass
class C:
    d: D = None


@dataclass
class B:
    c: C = None


@dataclass
class A:
    b: B = None



a = A(B(C(D(E("")))))

a.b.c.d.e.some_attr = ("Once upon a time there is"  " a village alongside the river that flows quitely.")

a.b.c.d.e = "Once upon a time there is"  " a village alongside the river that flows quitely."

a.b.c.d.e = "Once upon a time there is"		" a village alongside the river that flows quitely."  # two tabs

a.b.c.d.e = ("Once upon a time there is"  " a village alongside the river that flows quitely.")

a.b.c = "Once upon a time there is"   " a village alongside the river that flows quitely. QED"

a.b.c = "Once upon a time there is"   " a village alongside the river that flows quitely. The end."

abc = "this line has two parts; the first string is already long than the line limit of 88 characters;" " this is the 2nd part"

"It was the best of times, it was the worst of times, it was the age of wisdom, it was the age of foolishness, it was the epoch of belief, it was the epoch of incredulity, it was the season of Light, it was the season of Darkness, it was the spring of hope, it was the winter of despair, we had everything before us, we had nothing before us, we were all going direct to Heaven, we were all going direct the other way—in short, the period was so far like the present period, that some of its noisiest authorities insisted on its being received, for good or for evil, in the superlative degree of comparison only."


result_1 = (  # aaa
    "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
)

result_2 = (
    "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # bbb
)

result_3 = (
    "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
)  # ccc

result_4 = (  # aaa
    "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # bbb
)

result_5 = (
    "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # bbb
)  # ccc

result_6 = (  # aaa
    "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
)  # ccc

result_7 = (  # aaa
    "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # bbb
)  # ccc


(
    "my very long string that shouldn't get formatted even if it goes over the 88 char"
    " limit like it does now"
)

three_strings_1 = (
    "What if we have inline comments on "  # aaa
    "each line of a bad split? In that "  # bbb
    "case, should we just leave it alone?"  # ccc
)

three_strings_2 = (
    "a"  # aa
    "b"  # bb
    "c"  # cc
)


# output
from dataclasses import dataclass

variable_1 = (
    "very very very very very very very very very very very very very very long string"
)

variable_2 = "very very very very very very very very very very very very very very very long string"

variable_3 = (
    "very very very very very very very very very very very very very very long string"
)


@dataclass
class E:
    f: str = ""


@dataclass
class D:
    e: E = None


@dataclass
class C:
    d: D = None


@dataclass
class B:
    c: C = None


@dataclass
class A:
    b: B = None


a = A(B(C(D(E("")))))

a.b.c.d.e.some_attr = (
    "Once upon a time there is" " a village alongside the river that flows quitely."
)

a.b.c.d.e = (
    "Once upon a time there is" " a village alongside the river that flows quitely."
)

a.b.c.d.e = (
    "Once upon a time there is" " a village alongside the river that flows quitely."
)  # two tabs

a.b.c.d.e = (
    "Once upon a time there is" " a village alongside the river that flows quitely."
)

a.b.c = (
    "Once upon a time there is" " a village alongside the river that flows quitely. QED"
)

a.b.c = (
    "Once upon a time there is"
    " a village alongside the river that flows quitely. The end."
)

abc = (
    "this line has two parts; the first string is already long than the line limit of 88 characters;"
    " this is the 2nd part"
)

"It was the best of times, it was the worst of times, it was the age of wisdom, it was the age of foolishness, it was the epoch of belief, it was the epoch of incredulity, it was the season of Light, it was the season of Darkness, it was the spring of hope, it was the winter of despair, we had everything before us, we had nothing before us, we were all going direct to Heaven, we were all going direct the other way—in short, the period was so far like the present period, that some of its noisiest authorities insisted on its being received, for good or for evil, in the superlative degree of comparison only."


result_1 = (  # aaa
    "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
)

result_2 = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # bbb

result_3 = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # ccc

result_4 = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # aaa  # bbb

result_5 = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # bbb  # ccc

result_6 = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # aaa  # ccc

result_7 = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"  # aaa  # bbb  # ccc


(
    "my very long string that shouldn't get formatted even if it goes over the 88 char"
    " limit like it does now"
)

three_strings_1 = (
    "What if we have inline comments on "  # aaa
    "each line of a bad split? In that "  # bbb
    "case, should we just leave it alone?"  # ccc
)

three_strings_2 = "a" "b" "c"  # aa  # bb  # cc



# --function-definition-extra-indent = True
# --other-line-continuation-extra-indent = True
def function(
		arg1_with_very_long_name: int,
		arg2_with_very_long_name: float,
		arg3_with_very_long_name: list,
		):
	print('Hello world')


def func2(
		arg1_with_very_long_name: int,
		arg2_with_very_long_name: float,
		arg3_with_very_long_name: list,
		) -> None:
	print('Hello world')


def func3(a, b, c, d, e) -> int:
	return 2


def outer(
		arg1_with_very_long_name: int,
		*,
		arg2_with_very_long_name: float,
		arg3_with_very_long_name: list,
		) -> int:
	def middle(
			arg1_with_very_long_name: int,
			arg2_with_very_long_name: float,
			arg3_with_very_long_name: list,
			*args,
			):
		def inner(
				arg1_with_very_long_name: int,
				arg2_with_very_long_name: float,
				arg3_with_very_long_name: list,
				*args,
				**kwargs,
				):
			return 1

		print('middle layer')
		return 2

	return 3


class A:
	def __init__(
			self,
			arg1_with_very_long_name: int,
			arg2_with_very_long_name: float,
			*,
			arg3_with_very_long_name: list,
			) -> None:
		print('hello world')

	def outer(
			self,
			arg1_with_very_long_name: int,
			*,
			arg2_with_very_long_name: float,
			arg3_with_very_long_name: list,
			) -> int:
		def middle(
				arg1_with_very_long_name: int,
				arg2_with_very_long_name: float,
				arg3_with_very_long_name: list,
				*args,
				):
			def inner(
					arg1_with_very_long_name: int,
					arg2_with_very_long_name: float,
					arg3_with_very_long_name: list,
					*args,
					**kwargs,
					):
				class B:
					def __init__(
							self,
							arg1_with_very_long_name: int,
							arg2_with_very_long_name: float,
							arg3_with_very_long_name: list,
							*args,
							**kwargs,
							):
						print('initialized')

				return 1

			print('middle layer')
			return 2

		return 3


aa = [
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		]


bb = func1(
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		)


cc = {
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		value_with_very_very_very_very_very_very_long_name,
		123123123123123123123123123123123123123123123123123,
		'value_with_very_very_very_very_very_very_long_name',
		}


if (
		value_with_very_very_very_very_very_very_long_name
		and value_with_very_very_very_very_very_very_long_name
		and value_with_very_very_very_very_very_very_long_name
		and value_with_very_very_very_very_very_very_long_name
		and value_with_very_very_very_very_very_very_long_name
		and value_with_very_very_very_very_very_very_long_name
		and value_with_very_very_very_very_very_very_long_name
		):
	print(123)


some_val_with_very_long_name = (
		'very very very long string' + 'this is an even longer string'
		)



# --function-definition-extra-indent = False
# --other-line-continuation-extra-indent = False
def function(
    arg1_with_very_long_name: int,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
):
    print('Hello world')


def func2(
    arg1_with_very_long_name: int,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
) -> None:
    print('Hello world')


def func3(a, b, c, d, e) -> int:
    return 2


def outer(
    arg1_with_very_long_name: int,
    *,
    arg2_with_very_long_name: float,
    arg3_with_very_long_name: list,
) -> int:
    def middle(
        arg1_with_very_long_name: int,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
        *args,
    ):
        def inner(
            arg1_with_very_long_name: int,
            arg2_with_very_long_name: float,
            arg3_with_very_long_name: list,
            *args,
            **kwargs,
        ):
            return 1

        print('middle layer')
        return 2

    return 3


class A:
    def __init__(
        self,
        arg1_with_very_long_name: int,
        arg2_with_very_long_name: float,
        *,
        arg3_with_very_long_name: list,
    ) -> None:
        print('hello world')

    def outer(
        self,
        arg1_with_very_long_name: int,
        *,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
    ) -> int:
        def middle(
            arg1_with_very_long_name: int,
            arg2_with_very_long_name: float,
            arg3_with_very_long_name: list,
            *args,
        ):
            def inner(
                arg1_with_very_long_name: int,
                arg2_with_very_long_name: float,
                arg3_with_very_long_name: list,
                *args,
                **kwargs,
            ):
                class B:
                    def __init__(
                        self,
                        arg1_with_very_long_name: int,
                        arg2_with_very_long_name: float,
                        arg3_with_very_long_name: list,
                        *args,
                        **kwargs,
                    ):
                        print('initialized')

                return 1

            print('middle layer')
            return 2

        return 3


aa = [
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
]


bb = func1(
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
)


cc = {
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    value_with_very_very_very_very_very_very_long_name,
    123123123123123123123123123123123123123123123123123,
    'value_with_very_very_very_very_very_very_long_name',
}


if (
    value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
    and value_with_very_very_very_very_very_very_long_name
):
    print(123)


some_val_with_very_long_name = (
    'very very very long string' + 'this is an even longer string'
)



import pytest

import cercis
from cercis.utils_line_wrapping import check_eligibility_to_opt_out_of_line_wrapping

str1 = 'variable = "very very very very very very very very very very very very very very long string"'  # noqa: B950
str2 = 'some_class.a.b.some_variable_name = ("Once upon a time there is"  " a village alongside the river that flows quitely.")'  # noqa: B950


@pytest.mark.parametrize(
    "src_in, expected",
    [
        ("a = 2", False),
        ('a = "hello"', True),
        ("a = 'hello'", True),
        ('a = ("hello")', True),
        ('a = (("hello"))', True),
        ('a = ((("hello")))', True),
        ('a = (((((((((("hello"))))))))))', True),
        ('a = (((((((((("hello"))))))))), )', False),
        ('a = ("hello") ("world") ("good") ("morning")', False),
        ('a = "hello" + "world"', False),
        ('a = "hello""world"', True),
        ('a = "hello" "world"', True),
        ('a = "hello", "world"', False),
        ('a = "hello"  "world"', True),
        ('a = "hello"\t"world"', True),
        ('a = "hello" \t \t "world"', True),
        ('a = "hello" "world" "hello" "world" "hello"\t"world"  "hello"', True),
        ('a["key"].field[0] = "hello"', False),
        ('a[-1] = "hello"', False),
        ('func(thing) = "hello"', False),
        ('a.get("key") = "hello"', False),
        ('a.b.c.d.e.f.g.h.i.j.k.l.m.n = "hello"', True),
        ('a.b.c.d.e.f.g.h.i.j.k.l.m.n[0] = "hello"', False),
        ('a.b.c.d.e.f.g.h.i.j.k.l.m.n() = "hello"', False),
        ('a.b.c.d.e.f.g.h.i.j.k.l.m.n.__str__ = "hello"', True),
        (str1, True),
        (str2, True),
        ('"a string"', True),
        ('("another string")', True),
        ('(("a third string"))', False),
        ('(("a third string", ), )', False),
        ("12345678901234567890123456789012345678901234567890", False),
        ("3.14159265358979323846", False),
        ("pi = 3.14159265358979323846", False),
        ("pi = '3.14159265358979323846'", True),
        ('a = "hello",', False),
    ],
)
def test_check_line_eligibility_to_opt_out_of_line_wrapping(
        src_in: str,
        expected: bool,
) -> None:
    mode = cercis.Mode()
    src_node = cercis.lib2to3_parse(src_in.lstrip(), mode.target_versions)
    line_generator = cercis.LineGenerator(mode=mode, features=[])
    lines = list(line_generator.visit(src_node))
    assert len(lines) == 1

    eligible = check_eligibility_to_opt_out_of_line_wrapping(
        lines[0],
        wrap_line_with_long_string=False,
    )
    assert eligible == expected



from enum import Enum, auto
from typing import TYPE_CHECKING, Tuple

if TYPE_CHECKING:
    from cercis import Mode

SPACE: str = " "
TAB: str = "\t"
TWO_TABS: str = "\t\t"


class Indent(Enum):
    """
    A class to represent an indentation, which can belong to 1 of 4 types.

    Here is a brief explanation of these 4 types:
        - Dedent: It's actually the opposite of "indent"; we put it here
                  just because we'd like for all indent/dedent types to
                  be in the same place
        - Block: The indentation that should happen within a code block
                 (such as in a class, a function, a for loop, an if loop, ...)
        - Function definition continuation:
                When the "def function_name(...)" becomes too long, we wrap
                the list of arguments in new lines. This is one type of line
                continuation, which corresponds to this type of indentation.
        - Other line continuation:
                Other types of line continuation, such as when
                result = my_function(1, 2, 3, 4, 5, ...) becomes too long
                and we have to wrap this line. The corresponding indentation
                falls under this type.

    The implementation of this class is inspired by a similar implementation
    in Pyink -- in particular, here: https://github.com/google/pyink/blob/f93771c02e9a26ce9508c59d69c9337c95797eac/src/pyink/lines.py#L52-L61  # noqa: B950

    Pyink is a formatter that forks from Black. It inherits Black's MIT license.
    """

    DEDENT = auto()
    BLOCK = auto()
    FUNCTION_DEF_CONTINUATION = auto()
    OTHER_LINE_CONTINUATION = auto()

    def render(
            self,
            mode: "Mode",
            for_width_calculation: bool = False,
    ) -> str:
        """Render this indentation into actual characters.

        Args:
            mode:
                The global configuration of Cercis
            for_width_calculation:
                If True, we are rendering this indentation to calculate the
                width of the current line (if width > length limit, wrap line).
                If False, we are rendering this indentation to be included
                in the result.
        """
        if self == Indent.DEDENT:
            raise ValueError("Internal error: this method is invalid for DEDENT")

        if mode.use_tabs:
            if self == Indent.OTHER_LINE_CONTINUATION:
                ch = TWO_TABS if mode.other_line_continuation_extra_indent else TAB
            elif self == Indent.FUNCTION_DEF_CONTINUATION:
                ch = TWO_TABS if mode.function_definition_extra_indent else TAB
            else:
                ch = TAB

            if not for_width_calculation:
                return ch

            spaces_for_one_tab = " " * mode.tab_width
            return ch.replace(TAB, spaces_for_one_tab)

        spaces = SPACE * mode.base_indentation_spaces
        if self == Indent.OTHER_LINE_CONTINUATION:
            return spaces * 2 if mode.other_line_continuation_extra_indent else spaces

        if self == Indent.FUNCTION_DEF_CONTINUATION:
            return spaces * 2 if mode.function_definition_extra_indent else spaces

        return spaces


class MultipleIndents:
    """
    A class to hold multiple indentations.

    Attributes:
        indents:
            A collection of indents on a particular line of code
        mode:
            Global configuration, which contains information to determine
            how to render the indents into actual characters, or how to
            calculate the width of the indents
    """

    def __init__(self, indents: Tuple[Indent, ...], mode: "Mode") -> None:
        self.indents = indents
        self.mode = mode

    def render(self, for_width_calculation: bool = False) -> str:
        """Render the indents as actual characters.

        Args:
            for_width_calculation:
                If True, we are rendering this indentation to calculate the
                width of the current line (if width > length limit, wrap line).
                If False, we are rendering this indentation to be included
                in the formatting output.
        """
        return "".join(_.render(self.mode, for_width_calculation) for _ in self.indents)

    def calc_total_width(self) -> int:
        """Calculate the width of all the indents. We are not using len()
        because we can't simply treat a tab ('\t') as width 1
        when rendering."""
        chars_to_render: str = self.render(for_width_calculation=True)
        return len(chars_to_render)



# --function-definition-extra-indent = False
# --other-line-continuation-extra-indent = False
def function(
   arg1_with_very_long_name: int,
   arg2_with_very_long_name: float,
   arg3_with_very_long_name: list,
):
   print('Hello world')


def func2(
   arg1_with_very_long_name: int,
   arg2_with_very_long_name: float,
   arg3_with_very_long_name: list,
) -> None:
   print('Hello world')


def func3(a, b, c, d, e) -> int:
   return 2


def outer(
   arg1_with_very_long_name: int,
   *,
   arg2_with_very_long_name: float,
   arg3_with_very_long_name: list,
) -> int:
   def middle(
      arg1_with_very_long_name: int,
      arg2_with_very_long_name: float,
      arg3_with_very_long_name: list,
      *args,
   ):
      def inner(
         arg1_with_very_long_name: int,
         arg2_with_very_long_name: float,
         arg3_with_very_long_name: list,
         *args,
         **kwargs,
      ):
         return 1

      print('middle layer')
      return 2

   return 3


class A:
   def __init__(
      self,
      arg1_with_very_long_name: int,
      arg2_with_very_long_name: float,
      *,
      arg3_with_very_long_name: list,
   ) -> None:
      print('hello world')

   def outer(
      self,
      arg1_with_very_long_name: int,
      *,
      arg2_with_very_long_name: float,
      arg3_with_very_long_name: list,
   ) -> int:
      def middle(
         arg1_with_very_long_name: int,
         arg2_with_very_long_name: float,
         arg3_with_very_long_name: list,
         *args,
      ):
         def inner(
            arg1_with_very_long_name: int,
            arg2_with_very_long_name: float,
            arg3_with_very_long_name: list,
            *args,
            **kwargs,
         ):
            class B:
               def __init__(
                  self,
                  arg1_with_very_long_name: int,
                  arg2_with_very_long_name: float,
                  arg3_with_very_long_name: list,
                  *args,
                  **kwargs,
               ):
                  print('initialized')

            return 1

         print('middle layer')
         return 2

      return 3


aa = [
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
]


bb = func1(
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
)


cc = {
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   value_with_very_very_very_very_very_very_long_name,
   123123123123123123123123123123123123123123123123123,
   'value_with_very_very_very_very_very_very_long_name',
}


if (
   value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
   and value_with_very_very_very_very_very_very_long_name
):
   print(123)


some_val_with_very_long_name = (
   'very very very long string' + 'this is an even longer string'
)



import itertools
import re
from dataclasses import replace
from typing import Any, Iterator
from unittest.mock import patch

import pytest

import cercis
from cercis import TargetVersion
from tests.util import (
    DEFAULT_MODE,
    PY36_VERSIONS,
    all_data_cases,
    assert_format,
    dump_to_stderr,
    read_data,
)


def _override_single_quote_for_cleaner_future_rebase(mode: cercis.Mode) -> None:
    mode.single_quote = False


def _use_line_length_of_88_for_cleaner_future_rebase(mode: cercis.Mode) -> None:
    mode.line_length = 88


@pytest.fixture(autouse=True)
def patch_dump_to_file(request: Any) -> Iterator[None]:
    with patch("cercis.dump_to_file", dump_to_stderr):
        yield


def check_file(
        subdir: str, filename: str, mode: cercis.Mode, *, data: bool = True
) -> None:
    source, expected = read_data(subdir, filename, data=data)
    assert_format(source, expected, mode, fast=False)


@pytest.mark.filterwarnings("ignore:invalid escape sequence.*:DeprecationWarning")
@pytest.mark.parametrize("filename", all_data_cases("simple_cases"))
def test_simple_format(filename: str) -> None:
    magic_trailing_comma = filename != "skip_magic_trailing_comma"
    single_quote = filename == "expression"
    mode = cercis.Mode(
        magic_trailing_comma=magic_trailing_comma,
        wrap_line_with_long_string=True,
        collapse_nested_brackets=False,
        single_quote=single_quote,
        wrap_pragma_comments=True,
        line_length=88,
    )
    check_file("simple_cases", filename, mode)


@pytest.mark.parametrize("filename", all_data_cases("preview"))
def test_preview_format(filename: str) -> None:
    mode = cercis.Mode(
        preview=True,
        wrap_line_with_long_string=True,
        collapse_nested_brackets=False,
        wrap_pragma_comments=True,
        line_length=88,
    )
    _override_single_quote_for_cleaner_future_rebase(mode)
    check_file("preview", filename, mode)


def test_preview_context_managers_targeting_py38() -> None:
    source, expected = read_data("preview_context_managers", "targeting_py38.py")
    mode = cercis.Mode(preview=True, target_versions={cercis.TargetVersion.PY38})
    _override_single_quote_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode, minimum_version=(3, 8))


def test_preview_context_managers_targeting_py39() -> None:
    source, expected = read_data("preview_context_managers", "targeting_py39.py")
    mode = cercis.Mode(preview=True, target_versions={cercis.TargetVersion.PY39})
    _override_single_quote_for_cleaner_future_rebase(mode)
    _use_line_length_of_88_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode, minimum_version=(3, 9))


@pytest.mark.parametrize(
    "filename", all_data_cases("preview_context_managers/auto_detect")
)
def test_preview_context_managers_auto_detect(filename: str) -> None:
    match = re.match(r"features_3_(\d+)", filename)
    assert match is not None, "Unexpected filename format: %s" % filename
    source, expected = read_data("preview_context_managers/auto_detect", filename)
    mode = cercis.Mode(preview=True)
    _override_single_quote_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode, minimum_version=(3, int(match.group(1))))


# =============== #
# Complex cases
# ============= #


def test_empty() -> None:
    source = expected = ""
    assert_format(source, expected)


@pytest.mark.parametrize("filename", all_data_cases("py_36"))
def test_python_36(filename: str) -> None:
    source, expected = read_data("py_36", filename)
    mode = cercis.Mode(target_versions=PY36_VERSIONS)
    assert_format(source, expected, mode, minimum_version=(3, 6))


@pytest.mark.parametrize("filename", all_data_cases("py_37"))
def test_python_37(filename: str) -> None:
    source, expected = read_data("py_37", filename)
    mode = cercis.Mode(target_versions={cercis.TargetVersion.PY37})
    assert_format(source, expected, mode, minimum_version=(3, 7))


@pytest.mark.parametrize("filename", all_data_cases("py_38"))
def test_python_38(filename: str) -> None:
    source, expected = read_data("py_38", filename)
    mode = cercis.Mode(target_versions={cercis.TargetVersion.PY38})
    _override_single_quote_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode, minimum_version=(3, 8))


@pytest.mark.parametrize("filename", all_data_cases("py_39"))
def test_python_39(filename: str) -> None:
    source, expected = read_data("py_39", filename)
    mode = cercis.Mode(target_versions={cercis.TargetVersion.PY39})
    _override_single_quote_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode, minimum_version=(3, 9))


@pytest.mark.parametrize("filename", all_data_cases("py_310"))
def test_python_310(filename: str) -> None:
    source, expected = read_data("py_310", filename)
    mode = cercis.Mode(target_versions={cercis.TargetVersion.PY310})
    _override_single_quote_for_cleaner_future_rebase(mode)
    _use_line_length_of_88_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode, minimum_version=(3, 10))


@pytest.mark.parametrize("filename", all_data_cases("py_310"))
def test_python_310_without_target_version(filename: str) -> None:
    source, expected = read_data("py_310", filename)
    mode = cercis.Mode()
    _override_single_quote_for_cleaner_future_rebase(mode)
    _use_line_length_of_88_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode, minimum_version=(3, 10))


def test_patma_invalid() -> None:
    source, expected = read_data("miscellaneous", "pattern_matching_invalid")
    mode = cercis.Mode(target_versions={cercis.TargetVersion.PY310})
    with pytest.raises(cercis.parsing.InvalidInput) as exc_info:
        assert_format(source, expected, mode, minimum_version=(3, 10))

    exc_info.match("Cannot parse: 10:11")


@pytest.mark.parametrize("filename", all_data_cases("py_311"))
def test_python_311(filename: str) -> None:
    source, expected = read_data("py_311", filename)
    mode = cercis.Mode(target_versions={cercis.TargetVersion.PY311})
    _override_single_quote_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode, minimum_version=(3, 11))


@pytest.mark.parametrize("filename", all_data_cases("fast"))
def test_fast_cases(filename: str) -> None:
    source, expected = read_data("fast", filename)
    assert_format(source, expected, fast=True)


def test_python_2_hint() -> None:
    with pytest.raises(cercis.parsing.InvalidInput) as exc_info:
        assert_format("print 'daylily'", "print 'daylily'")
    exc_info.match(cercis.parsing.PY2_HINT)


@pytest.mark.filterwarnings("ignore:invalid escape sequence.*:DeprecationWarning")
def test_docstring_no_string_normalization() -> None:
    """Like test_docstring but with string normalization off."""
    source, expected = read_data("miscellaneous", "docstring_no_string_normalization")
    mode = replace(DEFAULT_MODE, string_normalization=False)
    assert_format(source, expected, mode)


def test_docstring_line_length_6() -> None:
    """Like test_docstring but with line length set to 6."""
    source, expected = read_data("miscellaneous", "linelength6")
    mode = cercis.Mode(line_length=6)
    assert_format(source, expected, mode)


def test_preview_docstring_no_string_normalization() -> None:
    """
    Like test_docstring but with string normalization off *and* the preview style
    enabled.
    """
    source, expected = read_data(
        "miscellaneous", "docstring_preview_no_string_normalization"
    )
    mode = replace(DEFAULT_MODE, string_normalization=False, preview=True)
    assert_format(source, expected, mode)


def test_long_strings_flag_disabled() -> None:
    """Tests for turning off the string processing logic."""
    source, expected = read_data("miscellaneous", "long_strings_flag_disabled")
    mode = replace(
        DEFAULT_MODE,
        experimental_string_processing=False,
        wrap_line_with_long_string=True,
    )
    _override_single_quote_for_cleaner_future_rebase(mode)
    assert_format(source, expected, mode)


def test_stub() -> None:
    mode = replace(DEFAULT_MODE, is_pyi=True)
    source, expected = read_data("miscellaneous", "stub.pyi")
    assert_format(source, expected, mode)


def test_nested_class_stub() -> None:
    mode = replace(DEFAULT_MODE, is_pyi=True, preview=True)
    source, expected = read_data("miscellaneous", "nested_class_stub.pyi")
    assert_format(source, expected, mode)


def test_power_op_newline() -> None:
    # requires line_length=0
    source, expected = read_data("miscellaneous", "power_op_newline")
    assert_format(source, expected, mode=cercis.Mode(line_length=0))


def test_type_comment_syntax_error() -> None:
    """Test that cercis is able to format python code with type comment syntax errors."""
    source, expected = read_data("type_comments", "type_comment_syntax_error")
    assert_format(source, expected)
    cercis.assert_equivalent(source, expected)


@pytest.mark.parametrize(
    "filename, extra_indent",
    [
        ("func_def_extra_indent.py", True),  # Cercis's default
        ("func_def_no_extra_indent.py", False),  # Black's default
    ],
)
def test_function_definition_extra_indent(filename: str, extra_indent: bool) -> None:
    mode = replace(
        DEFAULT_MODE,
        function_definition_extra_indent=extra_indent,
        # Adding trailing commas after *args etc. are only supported in py36+
        target_versions={TargetVersion.PY36},
    )
    check_file("configurable_cases/func_def_indent", filename, mode)


@pytest.mark.parametrize(
    "filename, extra_indent",
    [
        ("no_extra_indent.py", False),
        ("extra_indent.py", True),
    ],
)
def test_closing_bracket_extra_indent(filename: str, extra_indent: bool) -> None:
    mode = replace(
        DEFAULT_MODE,
        line_length=30,
        closing_bracket_extra_indent=extra_indent,
    )
    check_file("configurable_cases/closing_bracket_indent", filename, mode)


@pytest.mark.filterwarnings("ignore:invalid escape sequence.*:DeprecationWarning")
@pytest.mark.parametrize(
    "filename",
    all_data_cases("configurable_cases/single_quote"),
)
def test_single_quote(filename: str) -> None:
    mode = replace(
        DEFAULT_MODE,
        single_quote=True,
        wrap_line_with_long_string=True,
        collapse_nested_brackets=False,
        wrap_pragma_comments=True,
        line_length=88,
    )
    check_file("configurable_cases/single_quote", filename, mode)


@pytest.mark.parametrize(
    "filename, wrap_line",
    [
        ("test_cases__Cercis_default.py", False),
        ("test_cases__Black_default.py", True),
        ("long_strings_flag_disabled__Cercis_default.py", False),
        ("long_strings_flag_disabled__Black_default.py", True),
        ("edge_cases.py", False),
        ("edge_cases.py", True),
    ],
)
def test_opt_out_of_wrapping(filename: str, wrap_line: bool) -> None:
    mode = replace(DEFAULT_MODE, wrap_line_with_long_string=wrap_line)
    _override_single_quote_for_cleaner_future_rebase(mode)
    _use_line_length_of_88_for_cleaner_future_rebase(mode)
    check_file("configurable_cases/line_with_long_string", filename, mode)


@pytest.mark.parametrize(
    "filename, collapse_nested_brackets",
    [
        ("nested_brackets__Cercis_default.py", True),
        ("nested_brackets__Black_default.py", False),
        ("nested_brackets_explodes__Cercis_default.py", True),
        ("nested_brackets_explodes__Black_default.py", False),
    ],
)
def test_nested_brackets(filename: str, collapse_nested_brackets: bool) -> None:
    mode = replace(DEFAULT_MODE, collapse_nested_brackets=collapse_nested_brackets)
    _override_single_quote_for_cleaner_future_rebase(mode)
    check_file("configurable_cases/nested_brackets", filename, mode)


@pytest.mark.parametrize(
    "filename, wrap",
    [
        ("Cercis_default.py", False),
        ("Black_default.py", True),
    ],
)
def test_wrap_pragma_comments(filename: str, wrap: bool) -> None:
    mode = replace(DEFAULT_MODE, wrap_pragma_comments=wrap, line_length=80)
    check_file("configurable_cases/pragma_comments", filename, mode)


@pytest.mark.parametrize(
    "closing_bracket_extra_indent, base_indent_spaces, fdei, olcei",
    list(
        itertools.product(  # each list here corresponds to 1 argument above
            [False, True],
            [1, 2, 3, 4, 8],
            [False, True],
            [False, True],
        )
    ),
)
def test_indent_levels__use_spaces(
        closing_bracket_extra_indent: bool,
        base_indent_spaces: int,
        fdei: bool,
        olcei: bool,
) -> None:
    mode = replace(
        DEFAULT_MODE,
        line_length=80,
        base_indentation_spaces=base_indent_spaces,
        function_definition_extra_indent=fdei,
        other_line_continuation_extra_indent=olcei,
        closing_bracket_extra_indent=closing_bracket_extra_indent,
    )
    parent_folder = "configurable_cases/indentation/use_spaces"
    folder = (
        f"{parent_folder}/closing_bracket_extra_indent"
        if closing_bracket_extra_indent
        else f"{parent_folder}/closing_bracket_no_extra_indent"
    )
    check_file(
        f"{folder}/base_indent_spaces={base_indent_spaces}",
        f"fdei={fdei}_olcei={olcei}",
        mode,
    )


@pytest.mark.parametrize(
    "closing_bracket_extra_indent, fdei, olcei",
    list(
        itertools.product(  # each list here corresponds to 1 argument above
            [False, True],
            [False, True],
            [False, True],
        )
    ),
)
def test_indent_levels__use_tabs(
        closing_bracket_extra_indent: bool,
        fdei: bool,
        olcei: bool,
) -> None:
    mode = replace(
        DEFAULT_MODE,
        line_length=80,
        use_tabs=True,
        function_definition_extra_indent=fdei,
        other_line_continuation_extra_indent=olcei,
        closing_bracket_extra_indent=closing_bracket_extra_indent,
    )
    parent_folder = "use_tabs"
    folder = (
        f"{parent_folder}/closing_bracket_extra_indent"
        if closing_bracket_extra_indent
        else f"{parent_folder}/closing_bracket_no_extra_indent"
    )
    check_file(
        f"configurable_cases/indentation/{folder}",
        f"fdei={fdei}_olcei={olcei}",
        mode,
    )


@pytest.mark.parametrize(
    "tab_width",
    [1, 2, 3, 4, 5, 8],
)
def test_line_length_calculation_with_tabs(tab_width: int) -> None:
    mode = replace(
        DEFAULT_MODE,
        line_length=80,
        use_tabs=True,
        tab_width=tab_width,
        function_definition_extra_indent=False,
        other_line_continuation_extra_indent=False,
        closing_bracket_extra_indent=False,
    )
    filename = f"tab_width_{tab_width}"
    check_file(
        "configurable_cases/indentation/use_tabs/line_length_calculation",
        filename,
        mode,
    )



def function(arg1_with_very_long_name: int,
             arg2_with_very_long_name: float,
             arg3_with_very_long_name: list):
    print('Hello world')


def func2(arg1_with_very_long_name: int,
          arg2_with_very_long_name: float,
          arg3_with_very_long_name: list) -> None:
    print('Hello world')


def func3(a, b, c, d, e) -> int:
    return 2


def outer(arg1_with_very_long_name: int, *,
          arg2_with_very_long_name: float,
          arg3_with_very_long_name: list) -> int:
    def middle(arg1_with_very_long_name: int,
               arg2_with_very_long_name: float,
               arg3_with_very_long_name: list, *args):
        def inner(arg1_with_very_long_name: int,
                  arg2_with_very_long_name: float,
                  arg3_with_very_long_name: list, *args, **kwargs):
            return 1
        print('middle layer')
        return 2
    return 3


class A:
    def __init__(self, arg1_with_very_long_name: int,
          arg2_with_very_long_name: float,
          *,
          arg3_with_very_long_name: list) -> None:
        print('hello world')

    def outer(self, arg1_with_very_long_name: int, *,
          arg2_with_very_long_name: float,
          arg3_with_very_long_name: list) -> int:
        def middle(arg1_with_very_long_name: int,
                   arg2_with_very_long_name: float,
                   arg3_with_very_long_name: list, *args):
            def inner(arg1_with_very_long_name: int,
                      arg2_with_very_long_name: float,
                      arg3_with_very_long_name: list, *args, **kwargs):
                class B:
                    def __init__(self,
                                 arg1_with_very_long_name: int,
                                 arg2_with_very_long_name: float,
                                 arg3_with_very_long_name: list, *args, **kwargs):
                        print('initialized')
                return 1
            print('middle layer')
            return 2
        return 3


# output

def function(
        arg1_with_very_long_name: int,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
):
    print('Hello world')


def func2(
        arg1_with_very_long_name: int,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
) -> None:
    print('Hello world')


def func3(a, b, c, d, e) -> int:
    return 2


def outer(
        arg1_with_very_long_name: int,
        *,
        arg2_with_very_long_name: float,
        arg3_with_very_long_name: list,
) -> int:
    def middle(
            arg1_with_very_long_name: int,
            arg2_with_very_long_name: float,
            arg3_with_very_long_name: list,
            *args,
    ):
        def inner(
                arg1_with_very_long_name: int,
                arg2_with_very_long_name: float,
                arg3_with_very_long_name: list,
                *args,
                **kwargs,
        ):
            return 1

        print('middle layer')
        return 2

    return 3


class A:
    def __init__(
            self,
            arg1_with_very_long_name: int,
            arg2_with_very_long_name: float,
            *,
            arg3_with_very_long_name: list,
    ) -> None:
        print('hello world')

    def outer(
            self,
            arg1_with_very_long_name: int,
            *,
            arg2_with_very_long_name: float,
            arg3_with_very_long_name: list,
    ) -> int:
        def middle(
                arg1_with_very_long_name: int,
                arg2_with_very_long_name: float,
                arg3_with_very_long_name: list,
                *args,
        ):
            def inner(
                    arg1_with_very_long_name: int,
                    arg2_with_very_long_name: float,
                    arg3_with_very_long_name: list,
                    *args,
                    **kwargs,
            ):
                class B:
                    def __init__(
                            self,
                            arg1_with_very_long_name: int,
                            arg2_with_very_long_name: float,
                            arg3_with_very_long_name: list,
                            *args,
                            **kwargs,
                    ):
                        print('initialized')

                return 1

            print('middle layer')
            return 2

        return 3



def func():
    a = ''
    b = ""
    c = "'"
    d = "\""
    e = '""'
    f = "'"
    g = "hello""world"  # should be a space between 2 single quotes for readability
    h = "hello" "world"
    i = (
        "hello"
        'world'
    )
    j = (
        "hello_hello_hello_hello_hello_hello_hello_hello"
        "world_world_world_world_world_world_world_world_world"
    )



# output

def func():
    a = ''
    b = ''
    c = "'"
    d = '"'
    e = '""'
    f = "'"
    g = 'hello' 'world'  # should be a space between 2 single quotes for readability
    h = 'hello' 'world'
    i = 'hello' 'world'
    j = (
        'hello_hello_hello_hello_hello_hello_hello_hello'
        'world_world_world_world_world_world_world_world_world'
    )



# Line length limit: 30 chars,
# which is right here:       |


def function(arg1: int,
             arg2: float,
             arg3_with_very_long_name: list):
    print('Hello world')


def func2(arg1: int,
          arg2: float,
          arg3_with_very_long_name: list) -> None:
    print('Hello world')


result = func2(12345, 3.1415926, [1, 2, 3])

result = [123, 456, 789, -123, -456]

something = {'a': 1, 'b': 2, 'c': 3}

some_tuple = (1, 2, 3, 4, 5, 6, 7, 8)


class A:
    def __init__(self, a, b, c, d):
        print('hello world')


class B:
    def __init__(self, a, b, c, d,):
        print('hello world')


# Line length limit: 30 chars,
# which is right here:       |


# output


# Line length limit: 30 chars,
# which is right here:       |


def function(
        arg1: int,
        arg2: float,
        arg3_with_very_long_name: list,
):
    print('Hello world')


def func2(
        arg1: int,
        arg2: float,
        arg3_with_very_long_name: list,
) -> None:
    print('Hello world')


result = func2(
    12345,
    3.1415926,
    [1, 2, 3],
)

result = [
    123,
    456,
    789,
    -123,
    -456,
]

something = {
    'a': 1,
    'b': 2,
    'c': 3,
}

some_tuple = (
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
)


class A:
    def __init__(
            self, a, b, c, d
    ):
        print('hello world')


class B:
    def __init__(
            self,
            a,
            b,
            c,
            d,
    ):
        print('hello world')


# Line length limit: 30 chars,
# which is right here:       |



"""The test cases in this file comes from https://github.com/google/pyink,
which inherits Black's license (MIT license).
"""

WITH_SET = frozenset({
    1001,
    1002,
    1003,
    1004,
    1005,
})

WITH_LIST = frozenset([
    1001,
    1002,
    1003,
    1004,
    1005,
])

WITH_TUPLE = frozenset((
    1001,
    1002,
    1003,
    1004,
    1005,
))

MULTIPLE_LEVELS = array([[
    1001,
    1002,
    1003,
    1004,
    1005,
]])

TRAILING_COMMA = frozenset(
    {
        1001,
        1002,
        1003,
        1004,
        1005,
    },
)


# Make sure non-matching inner brackets are handled correctly.
PAIRS = [(1, 5), (1, 30), (1, 60), (2, 80)]


# When literal list/tuple/set/dict is used in functions and it fits on a single
# line, prefer breaking at outer brackets.
func_call_with_set = frozenset(
    {1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012}
)

func_call_with_list = frozenset(
    [1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012]
)

func_call_with_tuple = frozenset(
    (1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012)
)

func_call_with_dict = frozenset(
    {1001: 1002, 1003: 1004, 1005: 1006, 1007: 1008, 1009: 1010, 1011: 1012}
)

func_call_multiple_levels = array(
    [[1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012]]
)


# Make sure single- and two- item literals work as expected.
variable.method_call({
    "this_is_a_very_very_long_key": "this_is_a_very_very_very_long_value",
})

variable.method_call({
    "this_is_a_very_very_long_key1": "this_is_a_very_very_very_long_value1",
    "this_is_a_very_very_long_key2": "this_is_a_very_very_very_long_value2",
})

variable.method_call([
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
])

variable.method_call([
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
])

variable.method_call({
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
})

variable.method_call({
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
})

variable.method_call((
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
))

variable.method_call((
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
    "this_is_a_very_very_very_very_very_very_very_very_very_very_long_value",
))


class A:
    def foo():
        XXXXXXXXXXXX.append((
            "xxx_xxxxxxxxxx(xxxxx={}, xxxx={}, xxxxx, xxxx_xxxx_xxxxxxxxxx={})".format(
                xxxxx, xxxx, xxxx_xxxx_xxxxxxxxxx
            ),
            my_var,
            my_other_var,
        ))
